-- Execute by create_views.sh
-- SQL create new views

-- create new view [poi_all] for all POIs
CREATE VIEW geodata.poi_all AS
SELECT gid, poi.id, collected, EXTRACT('YEAR' FROM collected)::integer AS year, type.name as type, CONCAT(CONCAT(person.firstname, ' '), person.lastname) AS person, ST_X(geom) AS x, ST_Y(geom) AS y, ST_Z(geom) AS z, ST_Force2D(geom) as geom2d, geom as geom3d
FROM geodata.poi
LEFT JOIN geodata.person ON geodata.poi.personid = geodata.person.id
LEFT JOIN geodata.type ON geodata.poi.typeid = geodata.type.id ;

-- create new view [poi_year] the last POI on the years
CREATE VIEW geodata.poi_year AS
SELECT a.*
FROM geodata.poi_all a
LEFT JOIN geodata.poi_all b
  ON b.year = a.year
  AND b.collected > a.collected
  AND b.id = a.id
WHERE b.year IS NULL
ORDER BY a.id, a.year ;

-- create new view [poi_year_distance] for the yearly POIs with distance and a linestring to the next poi
CREATE VIEW geodata.poi_year_distance AS
SELECT a.*, b.collected AS collected_from, ST_Distance(a.geom3d::geography, b.geom3d::geography) AS distance, ST_MakeLine(b.geom3d, a.geom3d) AS geom_distance
FROM geodata.poi_year a
LEFT JOIN geodata.poi_year b
  ON b.id = a.id
  AND b.gid != a.gid
  AND b.collected < a.collected
  AND b.year >= (a.year - 1)
ORDER BY a.id, a.year ;

-- create new view [poi_year_distance_avg] for the yearly POIs with distance and a linestring to the next poi
CREATE VIEW geodata.poi_year_distance_avg AS
SELECT a.*, b.distance_max, b.distance_anz, b.distance_avg
FROM geodata.poi_year_distance a, 
  (SELECT id, SUM(distance) AS distance_max, COUNT(distance) AS distance_anz, AVG(distance) AS distance_avg
  FROM geodata.poi_year_distance
  GROUP BY id) b
WHERE a.id = b.id
ORDER BY a.id, a.year ;

-- create new view [poi_max] for the first and last POIs
CREATE VIEW geodata.poi_max AS
SELECT 2 AS nid, a.*
FROM geodata.poi_all a
LEFT JOIN geodata.poi_all b
  ON b.collected > a.collected
  AND b.id = a.id
WHERE b.collected IS NULL
UNION
SELECT 1 AS nid, a.*
FROM geodata.poi_all a
LEFT JOIN geodata.poi_all b
  ON b.collected < a.collected
  AND b.id = a.id
WHERE b.collected IS NULL ;

-- create new view [poi_max_distance] for the first and last POIs with distance and a linestring from first and last poi
CREATE VIEW geodata.poi_max_distance AS
SELECT a.*, b.collected AS collected_from, ST_Distance(a.geom3d::geography, b.geom3d::geography) AS distance, ST_MakeLine(a.geom3d, b.geom3d) AS geom_distance
FROM geodata.poi_max a
LEFT JOIN geodata.poi_max b
  ON a.id = b.id
  AND a.nid != b.nid
ORDER BY a.id, a.year ;

-- create new view [polygon_all] for all POLYGONs
CREATE VIEW geodata.polygon_all AS
SELECT gid, polygon.id, collected, EXTRACT('YEAR' FROM collected)::integer AS year, orientation, openingmax, openingmin, depth, CONCAT(CONCAT(person.firstname, ' '), person.lastname) AS person, ST_Area(geom::geography) AS area, ST_Perimeter(geom::geography) AS perimeter, ST_Force2D(geom) AS geom2d, geom AS geom3d, ST_Centroid(geom) AS geom_centroid, ST_X(ST_Centroid(geom)) AS centroid_x, ST_Y(ST_Centroid(geom)) AS centroid_y
FROM geodata.polygon
LEFT JOIN geodata.person ON geodata.polygon.personid = geodata.person.id ;

-- create new view [polygon_year] the last POLYGON on the years
CREATE VIEW geodata.polygon_year AS
SELECT a.*
FROM geodata.polygon_all a
LEFT JOIN geodata.polygon_all b
  ON b.year = a.year
  AND b.collected > a.collected
  AND b.id = a.id
WHERE b.year IS NULL
ORDER BY a.id, a.year ;

-- create new view [polygon_year_distance] for the yearly POLYGONs with centroid distance and a linestring from first and last centroid poi
CREATE VIEW geodata.polygon_year_distance AS
SELECT a.*, b.collected AS collected_from, ST_Distance(a.geom_centroid::geography, b.geom_centroid::geography) AS centroid_distance, ST_MakeLine(b.geom_centroid, a.geom_centroid) AS geom_centroid_distance, ST_Area(ST_Difference(a.geom3d, b.geom3d)::geography) AS difference_area, ST_Difference(a.geom3d, b.geom3d) AS geom_difference
FROM geodata.polygon_year a
LEFT JOIN geodata.polygon_year b
  ON b.id = a.id
  AND b.gid != a.gid
  AND b.collected < a.collected
  AND b.year >= (a.year - 1)
ORDER BY a.id, a.year ;

-- create new view [polygon_year_distance_avg] for the yearly POLYGONs with centroid distance and a linestring from first and last centroid poi
CREATE VIEW geodata.polygon_year_distance_avg AS
SELECT a.*, b.difference_area_max, b.difference_area_anz, b.difference_area_avg
FROM geodata.polygon_year_distance a, 
  (SELECT id, SUM(difference_area) AS difference_area_max, COUNT(difference_area) AS difference_area_anz, AVG(difference_area) AS difference_area_avg
  FROM geodata.polygon_year_distance
  GROUP BY id) b
WHERE a.id = b.id
ORDER BY a.id, a.year ;

-- create new view [polygon_max] for the first and last POLYGONs
CREATE VIEW geodata.polygon_max AS
SELECT 2 AS nid, a.*
FROM geodata.polygon_all a
LEFT JOIN geodata.polygon_all b
  ON b.collected > a.collected
  AND b.id = a.id
WHERE b.collected IS NULL
UNION
SELECT 1 AS nid, a.*
FROM geodata.polygon_all a
LEFT JOIN geodata.polygon_all b
  ON b.collected < a.collected
  AND b.id = a.id
WHERE b.collected IS NULL ;

-- create new view [polygon_max_distance] for the first and last POLYGONs with centroid distance and a linestring from first and last centroid poi and an intersect of both geometrys
CREATE VIEW geodata.polygon_max_distance AS
SELECT a.*, b.collected AS collected_from, ST_Distance(a.geom_centroid::geography, b.geom_centroid::geography) AS centroid_distance, ST_MakeLine(a.geom_centroid, b.geom_centroid) AS geom_centroid_distance, ST_Area(ST_Difference(b.geom3d, a.geom3d)::geography) AS difference_area, ST_Difference(b.geom3d, a.geom3d) AS geom_difference
FROM geodata.polygon_max a
LEFT JOIN geodata.polygon_max b
  ON a.id = b.id
  AND a.nid != b.nid
ORDER BY a.id, a.year ;
